local MakePlayerCharacter = require("prefabs/player_common")

local assets = {
	Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
	Asset("ANIM", "anim/ghost_abigale_build.zip"),
	Asset("ANIM", "anim/abigale.zip"),
	Asset("ATLAS", "images/saveslot_portraits/abigale.xml"),
	Asset("ATLAS", "bigportraits/abigale.xml"),
	Asset("ATLAS", "bigportraits/abigale_none.xml"),
	Asset("ATLAS", "images/map_icons/abigale.xml"),
	Asset("ATLAS", "images/avatars/avatar_abigale.xml"),
	Asset("ATLAS", "images/avatars/avatar_ghost_abigale.xml"),
	Asset("ATLAS", "images/avatars/self_inspect_abigale.xml"),
}

local prefabs = {}

local start_inv = {"abigail_flower", "nightmarefuel", "nightmarefuel"}

local function OnAttackWall(inst)
	local old = inst.replica.combat.IsAlly
	function inst.replica.combat:IsAlly(guy)
		if guy:HasTag("wall") then return true end
		return old(self, guy)
	end
end

local function ApplyUpgrades(inst)
	inst.components.locomotor.walkspeed = 6
	inst.components.locomotor.runspeed = 9
end

local function OnEat(inst, food)
	if food and food.components.edible then
		if food.prefab == "nightmarefuel" then
			local fx = SpawnPrefab("statue_transition")
			fx.entity:SetParent(inst.entity)
			fx.Transform:SetScale(.5,.5,.5)
			ApplyUpgrades(inst)
			inst.SoundEmitter:PlaySound("dontstarve/creatures/leif/livinglog_burn")
		end
	end
end

local function OnBecameRobot(inst)
	ApplyUpgrades(inst)
end

local function OnBecameGhost(inst)
	ApplyUpgrades(inst)
end

local function AutoAddNightMare(inst)
	local hat = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD) or nil
	if hat ~= nil and hat.prefab == "molehat" then
		local _fueled = hat.components.fueled
		if _fueled:GetPercent() <= 0.5 and inst.components.inventory:Has("nightmarefuel", 1) then
			SpawnPrefab("pandorachest_reset").entity:SetParent(inst.entity)
			_fueled:DoDelta(TUNING.LARGE_FUEL * _fueled.bonusmult)
			inst.components.inventory:ConsumeByName("nightmarefuel",1)
			if _fueled.ontakefuelfn then _fueled.ontakefuelfn(hat) end
		end
	end
	local weapon = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) or nil
	if weapon ~= nil and weapon.prefab == "hambat" then
		local _perishable = weapon.components.perishable
		if _perishable:GetPercent() <= 0.7 and inst.components.inventory:Has("nightmarefuel", 1) then
			SpawnPrefab("pandorachest_reset").entity:SetParent(inst.entity)
			_perishable:ReducePercent(-0.3)
			inst.components.inventory:ConsumeByName("nightmarefuel",1)
			if _perishable.perishfn then _perishable.perishfn(weapon) end
		end
	end
end

local function UpdataWorldState(inst)
	if TheWorld.state.isday then
		inst.components.combat.damagemultiplier = 0.50
		inst.components.health.absorb = -0.50
	elseif TheWorld.state.isdusk then
		inst.components.combat.damagemultiplier = 0.75
		inst.components.health.absorb = -0.25
	elseif TheWorld.state.isnight then
		inst.components.combat.damagemultiplier = 1.00
		inst.components.health.absorb = 0.00
	end
	if TheWorld.state.isfullmoon then
		inst:DoTaskInTime(1, function(inst)
			if not inst.components.health:IsDead() then
				inst.components.health:DoDelta(-2, nil, "fullmoon", true, nil, true)
			end
		end)
	end
end

local function UpdataHelmet(inst)
	local helmet = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD)
	if helmet and helmet.components.armor then
		helmet.components.armor:SetAbsorption(0)
	end
end

local function OnEquip(inst, data)
	if data.eslot == EQUIPSLOTS.HEAD then
		if data.item.components.armor then
			local helmet = data.item
			helmet.absorption = helmet.components.armor.absorb_percent
		end
	elseif data.eslot == EQUIPSLOTS.HANDS then
		if data.item.components.fishingrod then
			local fishrod = data.item
			fishrod.mintime = fishrod.components.fishingrod.minwaittime
			fishrod.maxtime = fishrod.components.fishingrod.maxwaittime
			fishrod.components.fishingrod:SetWaitTimes(1, 1)
		end
		if data.item.components.tool then
			if data.item.prefab == "goldenaxe" then
				local CHOP_tool = data.item
				CHOP_tool.CHOP_effectiveness = CHOP_tool.components.tool.actions[ACTIONS.CHOP]
				CHOP_tool.components.tool:SetAction(ACTIONS.CHOP, 5)
			elseif data.item.prefab == "goldenpickaxe" then
				local MINE_tool = data.item
				MINE_tool.MINE_effectiveness = MINE_tool.components.tool.actions[ACTIONS.MINE]
				MINE_tool.components.tool:SetAction(ACTIONS.MINE, 2)
			end
		end
	end
end

local function OnUnequip(inst, data)
	if data.eslot == EQUIPSLOTS.HEAD then
		if data.item.components.armor then
			local helmet = data.item
			helmet.components.armor:SetAbsorption(helmet.absorption)
		end
	elseif data.eslot == EQUIPSLOTS.HANDS then
		if data.item.components.fishingrod then
			local fishrod = data.item
			fishrod.components.fishingrod:SetWaitTimes(fishrod.mintime, fishrod.maxtime)
		end
		if data.item.components.tool then
			if data.item.prefab == "goldenaxe" then
				local CHOP_tool = data.item
				CHOP_tool.components.tool:SetAction(ACTIONS.CHOP, CHOP_tool.CHOP_effectiveness)
			elseif data.item.prefab == "goldenpickaxe" then
				local MINE_tool = data.item
				MINE_tool.components.tool:SetAction(ACTIONS.MINE, MINE_tool.MINE_effectiveness)
			end
		end
	end
end

local function IsValidVictim(victim)
	return victim and not victim:HasTag("wall") and victim.components.health and victim.components.combat
end

local function OnKillOther(inst, data)
	local victim = data.victim
	if IsValidVictim(victim) then
		local time = victim.components.health.destroytime or 2
		local x, y, z = victim.Transform:GetWorldPosition()
		local pt = victim:GetPosition()
		local scale = (victim:HasTag("epic") and 1.1) or (victim:HasTag("largecreature") and 0.9) or (victim:HasTag("smallcreature") and 0.5) or 0.7
		local num_loot = (victim:HasTag("epic") and 4) or (victim:HasTag("largecreature") and 2) or (victim:HasTag("smallcreature") and 0) or 1
		local num_victim = victim.components.stackable and victim.components.stackable:StackSize() or 1
		inst:DoTaskInTime(time, function(inst, x, y, z, scale)
			local fx = SpawnPrefab("wathgrithr_spirit")
			fx.Transform:SetPosition(x, y, z)
			fx.Transform:SetScale(scale, scale, scale)
			if not victim.components.lootdropper then victim:AddComponent("lootdropper") end
			if num_loot == 0 then return end
			for i = 1, num_loot * num_victim do victim.components.lootdropper:SpawnLootPrefab("nightmarefuel", pt) end
		end, x, y, z, scale)
	end
end

local function OnFullMoon(inst)
	inst.components.talker:Say(TUNING.WILLBEATTACKED)
	inst:DoTaskInTime(5,function()
		inst.components.talker:Say(TUNING.READYTOFIGHT)
	end)
	inst:DoTaskInTime(9,function()
		inst.components.lootdropper:SpawnLootPrefab("worm", inst:GetPosition())
	end)
end

local function OnDespawn(inst)
	if inst.abigail then
		inst.abigail.components.lootdropper:SetLoot(nil)
		inst.abigail.components.health:SetInvincible(true)
		inst.abigail:PushEvent("death")
		inst.abigail:DoTaskInTime(25 * FRAMES, inst.abigail.Remove)
	end
end

local function OnSave(inst, data)
	data.abigail = inst.abigail and inst.abigail:GetSaveRecord()
end

local function OnLoad(inst, data)
	if data and data.abigail and inst.abigail == nil then
		local abigail = SpawnSaveRecord(data.abigail)
		if abigail then
			if inst.migrationpets then
				table.insert(inst.migrationpets, abigail)
			end
			abigail.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")
			abigail:LinkToPlayer(inst)
		end
	end
end

local function common_postinit(inst)
	inst.MiniMapEntity:SetIcon("abigale.tex")
	inst.soundsname = "willow"
	inst:AddTag("abigale")
	inst:AddTag("insomniac")
	inst:AddTag("bookbuilder")
	inst:AddTag("fastbuilder")
	inst:AddTag("ghostlyfriend")
end

local function master_postinit(inst)
	inst.abigail = nil
	OnAttackWall(inst)
	ApplyUpgrades(inst)
	inst.abigail_flowers = {}
	inst:AddComponent("reader")
	inst:AddComponent("lootdropper")
	inst.components.hunger:SetMax(200)
	inst.components.sanity:SetMax(120)
	inst.components.health:SetMaxHealth(160)
	inst.components.builder.science_bonus = 1
	inst.components.sanity.neg_aura_mult = 2.25
	inst.components.sanity.night_drain_mult = -0.25
	inst.components.hunger:SetRate(2 * TUNING.WILSON_HUNGER_RATE)
	inst.components.eater:SetCanEatNightMare()
	inst.components.eater:SetOnEatFn(OnEat)
	inst.components.eater.stale_health = -1
	inst.components.eater.spoiled_health = -2
	inst:ListenForEvent("ms_respawnedfromghost", OnBecameRobot)
	inst:ListenForEvent("ms_becameghost", OnBecameGhost)
	inst:ListenForEvent("hungerdelta", AutoAddNightMare)
	inst:ListenForEvent("hungerdelta", UpdataWorldState)
	inst:ListenForEvent("hungerdelta", UpdataHelmet)
	inst:ListenForEvent("equip", OnEquip)
	inst:ListenForEvent("unequip", OnUnequip)
	inst:ListenForEvent("killed", OnKillOther)
	inst:WatchWorldState("isfullmoon", OnFullMoon)
	inst.OnDespawn = OnDespawn
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
end

return MakePlayerCharacter("abigale", prefabs, assets, common_postinit, master_postinit, start_inv)
